/************************************************************************/
/************************************************************************/
/*                                                                      */
/*      This package of C++ wrappers is intended to be called from a    */
/*      FORTRAN program. The main purpose of the package is to mediate  */
/*      between the call-by-address and call-by-value conventions in    */
/*      the two languages. In most cases, the arguments of the C++      */
/*      routines and the wrappers are the same.                         */
/*                                                                      */
/*      The wrappers should be treated as C function calls.             */
/*                                                                      */
/* Note: This code is a header file to facilitte inlining.              */
/************************************************************************/
/************************************************************************/

#if defined(SPRNG_VERSION) && SPRNG_VERSION >= 4

#undef SPRNG_MPI

#include "sprng_cpp.h"

#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/resource.h>

extern "C"
{
   int cget_seed_rng(int **genptr);
   int cfree_rng(int **genptr);
   int cmake_new_seed(void);
   int * cinit_rng_sim(int *seed,  int *mult, int *gtype);
   int * cinit_rng(int * rng_type, int *gennum, int *total_gen, int *seed, int *length);
   int cspawn_rng(int **genptr,  int *nspawned, int **newGen);
   int cget_rn_int_sim(void);
   int cget_rn_int(int **genptr);
   float cget_rn_flt_sim(void);
   float cget_rn_flt(int **genptr);
   double cget_rn_dbl_sim(void);
   double cget_rn_dbl(int **genptr);
   int cpack_rng(int **genptr, char **buffer);
   int cpack_rng_simple(char **buffer);
   int * cunpack_rng(char *buffer, int *rng_type);
   int * cunpack_rng_simple(char *buffer, int *rng_type);
   int cprint_rng( int **genptr);
   int cprint_rng_simple(void);
   int cseed_mpi(void);
   
   #ifdef SPRNG_MPI

   int * cinit_rng_simmpi(int *seed, int *mult, int * rng_type);
   int cget_rn_int_simmpi(void);
   float cget_rn_flt_simmpi(void);
   double cget_rn_dbl_simmpi(void);

   #endif

   double ccpu_t(void);
 
}

double ccpu_t(void)
{
  double   current_time;

#ifdef RUSAGE_SELF
  struct rusage temp;

  getrusage(RUSAGE_SELF, &temp);

  current_time = (temp.ru_utime.tv_sec + temp.ru_stime.tv_sec +
		  1.0e-6*(temp.ru_utime.tv_usec + temp.ru_stime.tv_usec));

#elif defined(CLOCKS_PER_SEC)
  current_time = clock()/((double) CLOCKS_PER_SEC);

#else
  fprintf(stderr,"\nERROR: Timing routines not available\n\n");
  current_time = 0.0;
#endif

  return (current_time);
}

int cget_seed_rng(int **genptr)
{
  Sprng * ptr = (Sprng *) *genptr;
  
  return ptr->get_seed_rng();
}

int cfree_rng(int **genptr)
{
  Sprng * ptr = (Sprng *) *genptr;
  
  return ptr->free_rng();
}

int cmake_new_seed(void)
{
  return make_new_seed();
}

int * cinit_rng_sim(int *seed,  int *mult, int *gtype)
{
  return init_rng_simple(*seed, *mult, *gtype);
}

int * cinit_rng(int * rng_type, int *gennum, int *total_gen, int *seed, int *length)
{
  Sprng * ptr;
  int status =0;
  ptr = SelectType(* rng_type);
  status = ptr->init_rng(*gennum, *total_gen, *seed, *length);
  
  if (status == 1)
    return (int *) ptr;
  else
    return (int *) NULL;
}

int cspawn_rng(int **genptr,  int *nspawned, int **newGen)
{
  int i, n;
  Sprng ** tmpGen;
  Sprng * ptr = (Sprng *) *genptr;
  
  n =  ptr->spawn_rng(*nspawned, &tmpGen);
  
  for (i=0; i< n; i++) 
    newGen[i] = (int *) tmpGen[i];

  if(n != 0)
    delete [] tmpGen;
  
  return n;
}

int cget_rn_int_sim(void)
{
  return get_rn_int_simple();
}

int cget_rn_int(int **genptr)
{
  Sprng * ptr = (Sprng *) *genptr;

  return ptr->get_rn_int();
}

float cget_rn_flt_sim(void)
{
  return get_rn_flt_simple();
}

float cget_rn_flt(int **genptr)
{
  Sprng * ptr = (Sprng *) *genptr;
  
  return ptr->get_rn_flt();
}

double cget_rn_dbl_sim(void)
{
  return get_rn_dbl_simple();
}

double cget_rn_dbl(int **genptr)
{
  Sprng * ptr = (Sprng *) *genptr;

  return ptr->get_rn_dbl();
}

int cpack_rng(int **genptr, char **buffer)
{
  int size;
  char *temp;
  Sprng * ptr = (Sprng *) *genptr;

  size = ptr->pack_rng(&temp);

  if (NULL != (*buffer = (char*)malloc(size))){
     if(temp != NULL)
     {
	memcpy(*buffer,temp,size);
	delete [] temp;
     }
  }
  
  return size;
}

int cpack_rng_simple(char **buffer)
{
  int size;
  char *temp;

  size = pack_rng_simple(&temp);
  if (NULL != (*buffer = (char*)malloc(size))){
     if(temp != NULL)
     {
	memcpy(*buffer,temp,size);
	delete [] temp;
     }
  }
  return size;
}

int * cunpack_rng(char *buffer, int * rng_type)
{
  Sprng * ptr = SelectType(* rng_type);
  int status = ptr->unpack_rng(buffer);

  if (status == 1)
    return (int *) ptr;
  else
    return (int *) NULL;
}

int * cunpack_rng_simple(char *buffer, int * rng_type)
{
  return unpack_rng_simple(buffer, *rng_type);
}

int cprint_rng( int **genptr)
{
  Sprng * ptr = (Sprng *) *genptr;

  return ptr->print_rng();
}

int cprint_rng_simple(void)
{
  return print_rng_simple();
}

/************************************************************************/
/************************************************************************/
/*                                                                      */
/*      This package of C wrappers is intended to be called from a      */
/*      FORTRAN program. The main purpose of the package is to mediate  */
/*      between the call-by-address and call-by-value conventions in    */
/*      the two languages. In most cases, the arguments of the C        */
/*      routines and the wrappers are the same. There are two           */
/*      exceptions to this. The trivial exception is that the C number  */
/*      scheme of 0 thru N-1 is automatically converted to the FORTRAN  */
/*      scheme of 1 thru N, so when referring to a particular generator */
/*      the FORTRAN user should number as is natural to that language.  */
/*                                                                      */
/*                                                                      */
/*      The wrappers should be treated as C function calls.             */
/*                                                                      */
/************************************************************************/
/************************************************************************/


int cseed_mpi(void)
{
#ifdef SPRNG_MPI
  return make_new_seed_mpi();
#else
  return -1;
#endif
}

#ifdef SPRNG_MPI

int * cinit_rng_simmpi(int *seed, int *mult, int *rng_type)
{
  return init_rng_simple_mpi(*seed, *mult, *rng_type);
}

int cget_rn_int_simmpi(void)
{
  return get_rn_int_simple_mpi();
}

float cget_rn_flt_simmpi(void)
{
  return get_rn_flt_simple_mpi();
}

double cget_rn_dbl_simmpi(void)
{
  return get_rn_dbl_simple_mpi();
}

#endif


#endif /* defined(SPRNG_VERSION) && SPRNG_VERSION >= 4 */
